@@ -1,7 +1,7 @@
 import requests
 import csv
 
-from pyPreservica.common import AuthenticatedAPI, Thumbnail, CHUNK_SIZE, EntityType, HEADER_TOKEN, content_api_identifier_to_type
+from pyPreservica.common import AuthenticatedAPI, Thumbnail, CHUNK_SIZE, HEADER_TOKEN, content_api_identifier_to_type
 
 
 class ContentAPI(AuthenticatedAPI):
@@ -11,6 +11,10 @@ class ContentAPI(AuthenticatedAPI):
 
     """
 
+    def __init__(self, username=None, password=None, tenant=None, server=None, use_shared_secret=False):
+        super().__init__(username, password, tenant, server, use_shared_secret)
+        self.callback = None
+
     class SearchResult:
         def __init__(self, metadata, refs, hits, results_list, next_start):
             self.metadata = metadata
@@ -19,6 +23,9 @@ class ContentAPI(AuthenticatedAPI):
             self.results_list = results_list
             self.next_start = next_start
 
+    def search_callback(self, fn):
+        self.callback = fn
+
     def object_details(self, entity_type, reference):
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'application/json'}
         params = {'id': f'sdb:{entity_type.value}|{reference}'}
@@ -33,12 +40,13 @@ class ContentAPI(AuthenticatedAPI):
         else:
             print(f"object_details failed with error code: {request.status_code}")
             print(request.request.url)
-            raise SystemExit
+            raise RuntimeError(request.status_code, f"object_details failed with error code: {request.status_code}")
 
     def download(self, reference, filename):
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'application/octet-stream'}
         params = {'id': f'sdb:IO|{reference}'}
-        with requests.get(f'https://{self.server}/api/content/download', params=params, headers=headers, stream=True) as req:
+        with requests.get(f'https://{self.server}/api/content/download', params=params, headers=headers,
+                          stream=True) as req:
             if req.status_code == requests.codes.ok:
                 with open(filename, 'wb') as file:
                     for chunk in req.iter_content(chunk_size=CHUNK_SIZE):
@@ -54,7 +62,7 @@ class ContentAPI(AuthenticatedAPI):
             else:
                 print(f"download failed with error code: {req.status_code}")
                 print(req.request.url)
-                raise SystemExit
+                raise RuntimeError(req.status_code, f"download failed with error code: {req.status_code}")
 
     def thumbnail(self, entity_type, reference, filename, size=Thumbnail.LARGE):
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'application/octet-stream'}
@@ -81,7 +89,7 @@ class ContentAPI(AuthenticatedAPI):
             else:
                 print(f"thumbnail failed with error code: {req.status_code}")
                 print(req.request.url)
-                raise SystemExit
+                raise RuntimeError(req.status_code, f"thumbnail failed with error code: {req.status_code}")
 
     def indexed_fields(self):
         headers = {HEADER_TOKEN: self.token}
@@ -98,7 +106,7 @@ class ContentAPI(AuthenticatedAPI):
         else:
             print(f"indexed_fields failed with error code: {results.status_code}")
             print(results.request.url)
-            raise SystemExit
+            raise RuntimeError(results.status_code, f"indexed_fields failed with error code: {results.status_code}")
 
     def simple_search_csv(self, query: str = "%", csv_file="search.csv", *args):
         page_size = 50
@@ -144,6 +152,7 @@ class ContentAPI(AuthenticatedAPI):
             refs = list(json['value']['objectIds'])
             refs = list(map(lambda x: content_api_identifier_to_type(x), refs))
             hits = int(json['value']['totalHits'])
+
             for m_row, r_row in zip(metadata, refs):
                 results_map = dict()
                 results_map['xip.reference'] = r_row[1]
@@ -151,6 +160,11 @@ class ContentAPI(AuthenticatedAPI):
                     results_map[li['name']] = li['value']
                 results_list.append(results_map)
             next_start = start_index + page_size
+
+            if self.callback is not None:
+                value = str(f'{len(results_list) + start_index}:{hits}')
+                self.callback(value)
+
             search_results = self.SearchResult(metadata, refs, hits, results_list, next_start)
             return search_results
         elif results.status_code == requests.codes.unauthorized:
@@ -159,4 +173,77 @@ class ContentAPI(AuthenticatedAPI):
         else:
             print(f"search failed with error code: {results.status_code}")
             print(results.request.url)
-            raise SystemExit
+            raise RuntimeError(results.status_code, f"simple_search failed with error code: {results.status_code}")
+
+    def search_index_filter_csv(self, query: str = "%", csv_file="search.csv", map_fields=None):
+        page_size = 50
+        if map_fields is None:
+            map_fields = dict()
+        if "xip.reference" not in map_fields:
+            map_fields["xip.reference"] = ""
+
+        header_fields = list(map_fields.keys())
+        index = header_fields.index("xip.reference")
+        header_fields.insert(0, header_fields.pop(index))
+        with open(csv_file, newline='', mode="wt", encoding="utf-8") as csvfile:
+            writer = csv.DictWriter(csvfile, fieldnames=header_fields)
+            writer.writeheader()
+            writer.writerows(self.search_index_filter_list(query, page_size, map_fields))
+
+    def search_index_filter_list(self, query: str = "%", page_size: int = 10, map_fields=None):
+        search_result = self.search_index_filter(query, 0, page_size, map_fields)
+        for e in search_result.results_list:
+            yield e
+        found = len(search_result.results_list)
+        while search_result.hits > found:
+            search_result = self.search_index_filter(query, found, page_size, map_fields)
+            for e in search_result.results_list:
+                yield e
+            found = found + len(search_result.results_list)
+
+    def search_index_filter(self, query: str = "%", start_index: int = 0, page_size: int = 10, map_fields=None):
+        start_from = str(start_index)
+        headers = {'Content-Type': 'application/x-www-form-urlencoded', HEADER_TOKEN: self.token}
+
+        field_list = list()
+        for key, value in map_fields.items():
+            if value == "":
+                field_list.append('{' f' "name": "{key}", "values": [] ' + '}')
+            else:
+                field_list.append('{' f' "name": "{key}", "values": ["{value}"] ' + '}')
+
+        filter_terms = ','.join(field_list)
+
+        queryterm = ('{ "q":  "%s",  "fields":  [ %s ] }' % (query, filter_terms))
+
+        payload = {'start': start_from, 'max': str(page_size), 'metadata': list(map_fields.keys()), 'q': queryterm}
+        results = requests.post(f'https://{self.server}/api/content/search', data=payload, headers=headers)
+        results_list = list()
+        if results.status_code == requests.codes.ok:
+            json = results.json()
+            metadata = json['value']['metadata']
+            refs = list(json['value']['objectIds'])
+            refs = list(map(lambda x: content_api_identifier_to_type(x), refs))
+            hits = int(json['value']['totalHits'])
+
+            for m_row, r_row in zip(metadata, refs):
+                results_map = dict()
+                results_map['xip.reference'] = r_row[1]
+                for li in m_row:
+                    results_map[li['name']] = li['value']
+                results_list.append(results_map)
+            next_start = start_index + page_size
+
+            if self.callback is not None:
+                value = str(f'{len(results_list) + start_index}:{hits}')
+                self.callback(value)
+
+            search_results = self.SearchResult(metadata, refs, hits, results_list, next_start)
+            return search_results
+        elif results.status_code == requests.codes.unauthorized:
+            self.token = self.__token__()
+            return self.search_index_filter(query, start_index, page_size, map_fields)
+        else:
+            print(f"search failed with error code: {results.status_code}")
+            print(results.request.url)
+            raise RuntimeError(results.status_code, f"search_index_filter failed")
