@@ -8,6 +8,7 @@ from datetime import datetime
 from xml.dom import minidom
 from xml.etree import ElementTree
 from xml.etree.ElementTree import Element, SubElement
+from sanitize_filename import sanitize
 
 import boto3
 from boto3.s3.transfer import TransferConfig
@@ -136,31 +137,36 @@ def __make_representation_multiple_co__(xip, rep_name, rep_type, rep_files, io_r
     return refs_dict
 
 
-def cvs_to_cmis_xslt(csv_file, xml_namespace, root_element, title="Metadata Title", export_folder=None):
+def cvs_to_cmis_xslt(csv_file, xml_namespace, root_element, title="Metadata Title", export_folder=None,
+                     additional_namespaces=None):
     """
             Create a custom CMIS transform to display metadata within UA.
 
     """
-    headers = list()
+    headers = set()
     with open(csv_file, encoding='utf-8-sig', newline='') as csvfile:
         reader = csv.reader(csvfile)
         for row in reader:
             for header in row:
                 xml_tag = header.strip()
-                xml_tag = xml_tag.title()
                 xml_tag = xml_tag.replace(" ", "")
-                xml_tag = xml_tag.replace("_", "")
-                headers.append(xml_tag)
+                xml_tag = xml_tag.replace("-", "")
+                headers.add(xml_tag)
             break
 
-    xml_stylesheet = xml.etree.ElementTree.Element("xsl:stylesheet",
-                                                   {"version": "2.0",
-                                                    "xmlns:xsl": "http://www.w3.org/1999/XSL/Transform",
-                                                    "xmlns:fn": "http://www.w3.org/2005/xpath-functions",
-                                                    "xmlns:xs": "http://www.w3.org/2001/XMLSchema",
-                                                    "xmlns:csv": xml_namespace,
-                                                    "xmlns": "http://www.tessella.com/sdb/cmis/metadata",
-                                                    "exclude-result-prefixes": "csv"})
+    namespaces = {"version": "2.0",
+                  "xmlns:xsl": "http://www.w3.org/1999/XSL/Transform",
+                  "xmlns:fn": "http://www.w3.org/2005/xpath-functions",
+                  "xmlns:xs": "http://www.w3.org/2001/XMLSchema",
+                  "xmlns:csv": xml_namespace,
+                  "xmlns": "http://www.tessella.com/sdb/cmis/metadata",
+                  "exclude-result-prefixes": "csv"}
+
+    if additional_namespaces is not None:
+        for prefix, uri in additional_namespaces.items():
+            namespaces["xmlns:" + prefix] = uri
+
+    xml_stylesheet = xml.etree.ElementTree.Element("xsl:stylesheet", namespaces)
 
     xml.etree.ElementTree.SubElement(xml_stylesheet, "xsl:output", {"method": "xml", "indent": "yes"})
 
@@ -173,8 +179,15 @@ def cvs_to_cmis_xslt(csv_file, xml_namespace, root_element, title="Metadata Titl
 
     xml_templates = xml.etree.ElementTree.SubElement(xml_group, "xsl:apply-templates")
 
-    elements = "csv:"
-    elements = elements + "|csv:".join(headers)
+    elements = ""
+
+    for header in headers:
+        if ":" in header:
+            elements = elements + "|" + header
+        else:
+            elements = elements + "|csv:" + header
+
+    elements = elements[1:]
 
     xml_matches = xml.etree.ElementTree.SubElement(xml_stylesheet, "xsl:template", {"match": elements})
 
@@ -200,31 +213,52 @@ def cvs_to_cmis_xslt(csv_file, xml_namespace, root_element, title="Metadata Titl
     return cmis_xslt
 
 
-def cvs_to_xsd(csv_file, xml_namespace, root_element, export_folder=None):
+def cvs_to_xsd(csv_file, xml_namespace, root_element, export_folder=None, additional_namespaces=None):
     """
         Create a XSD definition based on the csv file
 
     """
-    headers = list()
+    headers = set()
     with open(csv_file, encoding='utf-8-sig', newline='') as csvfile:
         reader = csv.reader(csvfile)
         for row in reader:
             for header in row:
                 xml_tag = header.strip()
-                xml_tag = xml_tag.title()
                 xml_tag = xml_tag.replace(" ", "")
-                xml_tag = xml_tag.replace("_", "")
-                headers.append(xml_tag)
+                xml_tag = xml_tag.replace("-", "")
+                headers.add(xml_tag)
             break
-    xml_schema = xml.etree.ElementTree.Element("xs:schema", {"xmlns:xs": "http://www.w3.org/2001/XMLSchema",
-                                                             "attributeFormDefault": "unqualified",
-                                                             "elementFormDefault": "qualified",
-                                                             "targetNamespace": xml_namespace})
+
+    namespaces = {"xmlns:xs": "http://www.w3.org/2001/XMLSchema",
+                  "attributeFormDefault": "unqualified",
+                  "elementFormDefault": "qualified",
+                  "targetNamespace": xml_namespace}
+
+    if additional_namespaces is not None:
+        for prefix, uri in additional_namespaces.items():
+            namespaces["xmlns:" + prefix] = uri
+
+    xml_schema = xml.etree.ElementTree.Element("xs:schema", namespaces)
+
+    for prefix, namespace in additional_namespaces.items():
+        xml_import = xml.etree.ElementTree.SubElement(xml_schema, "xs:import", {"namespace": namespace})
+
     xml_element = xml.etree.ElementTree.SubElement(xml_schema, "xs:element", {"name": root_element})
+
     xml_complex_type = xml.etree.ElementTree.SubElement(xml_element, "xs:complexType")
     xml_sequence = xml.etree.ElementTree.SubElement(xml_complex_type, "xs:sequence")
     for header in headers:
-        xml_e = xml.etree.ElementTree.SubElement(xml_sequence, "xs:element", {"type": "xs:string", "name": header})
+        if ":" in header:
+            prefix, sep, tag = header.partition(":")
+            try:
+                namespace = additional_namespaces[prefix]
+                xml.etree.ElementTree.SubElement(xml_sequence, "xs:element",
+                                                 {"ref": header, "xmlns:" + prefix: namespace})
+            except KeyError:
+                xml.etree.ElementTree.SubElement(xml_sequence, "xs:element", {"type": "xs:string", "name": header})
+        else:
+            xml.etree.ElementTree.SubElement(xml_sequence, "xs:element", {"type": "xs:string", "name": header})
+
     xml_request = xml.etree.ElementTree.tostring(xml_schema, encoding='utf-8', xml_declaration=True)
 
     xsd_file = root_element + ".xsd"
@@ -236,21 +270,21 @@ def cvs_to_xsd(csv_file, xml_namespace, root_element, export_folder=None):
     return xsd_file
 
 
-def csv_to_search_xml(csv_file, xml_namespace, root_element, title="Metadata Title", export_folder=None):
+def csv_to_search_xml(csv_file, xml_namespace, root_element, title="Metadata Title", export_folder=None,
+                      additional_namespaces=None):
     """
         Create a custom Preservica search index based on the columns in a csv file
 
     """
-    headers = list()
+    headers = set()
     with open(csv_file, encoding='utf-8-sig', newline='') as csvfile:
         reader = csv.reader(csvfile)
         for row in reader:
             for header in row:
                 xml_tag = header.strip()
-                xml_tag = xml_tag.title()
                 xml_tag = xml_tag.replace(" ", "")
-                xml_tag = xml_tag.replace("_", "")
-                headers.append(xml_tag)
+                xml_tag = xml_tag.replace("-", "")
+                headers.add(xml_tag)
             break
 
     xml_index = xml.etree.ElementTree.Element("index", {"xmlns": "http://www.preservica.com/customindex/v1"})
@@ -265,11 +299,20 @@ def csv_to_search_xml(csv_file, xml_namespace, root_element, title="Metadata Tit
     xml_shortName.text = short_name
 
     for header in headers:
+        if ":" in header:
+            xpath_expression = f"//{short_name}:{root_element}/{header}"
+        else:
+            xpath_expression = f"//{short_name}:{root_element}/{short_name}:{header}"
+
         attr = {"indexName": header, "displayName": header,
-                "xpath": f"//{short_name}:{root_element}/{short_name}:{header}",
+                "xpath": xpath_expression,
                 "indexType": "STRING_DEFAULT"}
         xml_term = xml.etree.ElementTree.SubElement(xml_index, "term", attr)
 
+    if additional_namespaces is not None:
+        for prefix, uri in additional_namespaces.items():
+            xml.etree.ElementTree.SubElement(xml_index, "namespaceMapping",  {"key": prefix, "value": uri})
+
     xml_request = xml.etree.ElementTree.tostring(xml_index, encoding='utf-8', xml_declaration=True)
     search_xml = root_element + "-index.xml"
     if export_folder is not None:
@@ -280,7 +323,8 @@ def csv_to_search_xml(csv_file, xml_namespace, root_element, title="Metadata Tit
     return search_xml
 
 
-def cvs_to_xml(csv_file, xml_namespace, root_element, file_name_column="filename", export_folder=None):
+def cvs_to_xml(csv_file, xml_namespace, root_element, file_name_column="filename", export_folder=None,
+               additional_namespaces=None):
     """
     Export the rows of a CSV file as XML metadata documents which can be added to Preservica assets
 
@@ -296,15 +340,18 @@ def cvs_to_xml(csv_file, xml_namespace, root_element, file_name_column="filename
                 if header == file_name_column:
                     link_column_id = col_id
                 xml_tag = header.strip()
-                xml_tag = xml_tag.title()
                 xml_tag = xml_tag.replace(" ", "")
-                xml_tag = xml_tag.replace("_", "")
+                xml_tag = xml_tag.replace("-", "")
                 headers.append(xml_tag)
             break
         if link_column_id > 0:
+            namespaces = {"xmlns": xml_namespace}
+            if additional_namespaces is not None:
+                for prefix, uri in additional_namespaces.items():
+                    namespaces["xmlns:" + prefix] = uri
             for row in reader:
                 col_id = 0
-                xml_object = xml.etree.ElementTree.Element(root_element, {"xmlns": xml_namespace})
+                xml_object = xml.etree.ElementTree.Element(root_element, namespaces)
                 for value, header in zip(row, headers):
                     col_id += 1
                     xml.etree.ElementTree.SubElement(xml_object, header).text = value
@@ -312,6 +359,7 @@ def cvs_to_xml(csv_file, xml_namespace, root_element, file_name_column="filename
                         file_name = value
                 xml_request = xml.etree.ElementTree.tostring(xml_object, encoding='utf-8', xml_declaration=True)
                 name = file_name + ".xml"
+                name = sanitize(name)
                 if export_folder is not None:
                     name = os.path.join(export_folder, name)
                 file = open(name, "wt", encoding="utf-8")
@@ -388,7 +436,8 @@ def complex_asset_package(preservation_files_list=None, access_files_list=None,
         for content_ref, filename in preservation_refs_dict.items():
             default_content_objects_title = os.path.splitext(os.path.basename(filename))[0]
             preservation_content_title = kwargs.get('Preservation_Content_Title', default_content_objects_title)
-            preservation_content_description = kwargs.get('Preservation_Content_Description',  default_content_objects_title)
+            preservation_content_description = kwargs.get('Preservation_Content_Description',
+                                                          default_content_objects_title)
 
             if isinstance(preservation_content_title, dict):
                 preservation_content_title = preservation_content_title[filename]
